package l10_concurrent;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import static java.util.concurrent.TimeUnit.SECONDS;

public class BeeperControl {

	public static void beepForSomeTime(int duration) {
		final long nano = 1000000000L;
		final long timeStart = System.nanoTime() / nano;

		final Runnable beeper = new Runnable() {
			@Override
			public void run() {
				System.out.println("beep at " + (System.nanoTime() / nano - timeStart));
			}
		};

//		The methods ScheduledExecutorService#scheduleAtFixedRate() and
//		ScheduledExecutorService#scheduleWithFixedDelay() repeat the task
//		periodically and both return ScheduledFuture<?>, that means the
//		get() method will always return null for them. These futures,
//		however, can be cancelled based on some condition.

		final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
		final ScheduledFuture<?> scheduledFuture = scheduler.scheduleAtFixedRate(
			beeper, 1, 1, SECONDS);

		//	Timeout after 'duration' seconds / cancel the future & shut down the scheduler
		new Timeout(1000 * duration, "Timeout", scheduler, scheduledFuture);
	}

	public static void main(String[] args) {
		int duration = (args.length == 0) ? 5 : Integer.parseInt(args[0]);
		BeeperControl.beepForSomeTime(duration);
	}
}
