package l10_concurrent;

import utilities.RandomGenerators;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;

public class CallableFutures {

//	A Future function returns immediately, returning a lightweight Future<Long>
//	object. This object is a promise that Long will be available in the future.
//	You can then retrieve it using Future.get(). In other words Future is a
//	proxy or a wrapper around an object that is not yet there. Once the
//	asynchronous computation is done, you can extract it.

//	Future.get() is the most important method. It blocks and waits until
//	promised result is available (resolved). So if we really need that Long,
//	just call get() and wait. There is an overloaded version that accepts
//	timeout, so you won't wait forever if something goes wild. TimeoutException
//	is thrown if waiting for too long.

//	The submit(Callable) method is similar to the submit(Runnable) method
//	except for the type of parameter it takes. The Callable instance is very
//	similar to a Runnable except that its call() method can return a result.
//	The Callable's result can be obtained via the Future object returned by the
//	submit(Callable) method.

	private static final int WORKERS = 20, MAX = 100;

	public static void main(String[] args) {
		int numberOfThreads = Runtime.getRuntime().availableProcessors();
		Set<Future<Pair>> list = new HashSet<>();
		try (ExecutorService executor = Executors.newFixedThreadPool(numberOfThreads)) {
			for (int i = 0; i < WORKERS; i++) {
				list.add(executor.submit(new MyCallable(MAX, i)));
			}
		}

		long sum = 0;
		for (Future<Pair> future : list) {
			try {
				Pair res = future.get(); // blocks until result available
				System.out.println(res);
				sum += res.sum;
			} catch (InterruptedException | ExecutionException e) {
				e.printStackTrace();
			}
		}
		System.out.println("sum = " + sum);
	}

	record Pair(long sum, int threadId) {
		@Override
		public String toString() {
			return String.format("threadId: %2d, sum: %4d", threadId, sum);
		}
	}

	record MyCallable(int max, int threadId) implements Callable<Pair> {
		@Override
		public Pair call() {
			long sum = 0;
			for (long i = 0; i <= max; i++) {
				sum += RandomGenerators.getIntThread(100);
			}
			return new Pair(sum, threadId);
		}
	}
}
