package l10_concurrent;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class ConcurrentLinkedQueueOptimal {
	public static void main(String[] args) {
		int numberOfProducers = (args.length > 0) ? Integer.parseInt(args[0]) : 4;
		int numberOfThreads = Runtime.getRuntime().availableProcessors();
		ConcurrentLinkedQueue<Integer> queue = new ConcurrentLinkedQueue<>();
		final CountDownLatch latch = new CountDownLatch(numberOfProducers);

		// Producer threads
		try (ExecutorService executorService = Executors.newFixedThreadPool(numberOfThreads)) {
			for (int i = 0; i < numberOfProducers; i++) {
				executorService.submit(new Producer("Producer " + i, queue, latch));
			}

			// Wait for Producers to finish
			ThreadHelperMethods.awaitCountDownLatch(latch);
			for (int i = 0; i < numberOfProducers; i++) {
				executorService.submit(new Consumer("Consumer " + i, queue));
			}
		}
	}

	record Producer(String name, ConcurrentLinkedQueue<Integer> queue, CountDownLatch latch) implements Runnable {
		@Override
		public void run() {
			try {
				for (int i = 0; i < 5; i++) {
					ThreadHelperMethods.safeSleep(500);
					int element = RandomGenerators.getIntThread(100);
					queue.add(element);
					System.out.println(name + " added: " + element);
				}
			} finally {
				latch.countDown();
			}
		}
	}

	record Consumer(String name, ConcurrentLinkedQueue<Integer> queue) implements Runnable {
		@Override
		public void run() {
			int count = 0;
			Integer number;
			while ((number = queue.poll()) != null) {
				ThreadHelperMethods.safeSleep(1000);
				System.out.println("\t" + name + " removed: " + number);
				count++;
			}
			System.out.println(name + " got " + count + " elements");
		}
	}
}

