package l10_concurrent;

import utilities.ThreadHelperMethods;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/*
	 A java.util.concurrent.CountDownLatch is a concurrency construct that
	 allows one or more threads to wait for a given set of operations to
	 complete.

	 A CountDownLatch is initialized with a given count. This count is
	 decremented by calls to the countDown() method. Threads waiting for this
	 count to reach zero can call one of the await() methods. Calling await()
	 blocks the thread until the count reaches zero.
*/

public class CountDownLatchSimple {

	public static void main(String[] args) {
		int numberThreads = Runtime.getRuntime().availableProcessors();

		// Latch opens when all threads are done
		final CountDownLatch completedThreadCounter = new CountDownLatch(numberThreads);

		try (ExecutorService executorService = Executors.newFixedThreadPool(numberThreads)) {
			for (int i = 0; i < numberThreads; ++i) {
				executorService.submit(new CDLThread("Thread " + i, completedThreadCounter));
			}

			// wait for the threads to finish
			ThreadHelperMethods.awaitCountDownLatch(completedThreadCounter);
			System.out.println("All threads completed");
		}
	}

	record CDLThread(String name, CountDownLatch completedThreadCounter) implements Runnable {

		@Override
		public void run() {
			System.out.println("\t" + name + " starting");
			ThreadHelperMethods.safeSleep(1000, 4000);
			System.out.println("\t\t" + name + " finished");
			// thread ready; decrement latch counter.
			completedThreadCounter.countDown(); // thread finished
		}
	}
}

