package l10_concurrent;

import java.util.concurrent.CyclicBarrier;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

//	The java.util.concurrent.CyclicBarrier class is a synchronization mechanism
//	that can synchronize threads progressing through some algorithm. In other
//	words, it is a barrier that all threads must wait at, until all threads
//	reach it, before any of the threads can continue.

public class CyclicBarrierExample {

	public static void main(String[] args) {

		int numberThreads = (args.length == 0) ? 4 : Integer.parseInt(args[0]);
		int n = Runtime.getRuntime().availableProcessors();
		CyclicBarrier barrier = new CyclicBarrier(numberThreads);

		try (ExecutorService executorService = Executors.newFixedThreadPool(n)) {
			// create threads passing a name, sleep time, and cyclic barrier
			for (int i = 1; i <= numberThreads; ++i) {
				executorService.submit(new CBRunnable("Thread " + i, 1000 * i, barrier));
			}
		}
	}

	record CBRunnable(String name, int sleep, CyclicBarrier barrier) implements Runnable {
		@Override
		public void run() {
			try {
				System.out.println(name + ": started");
				System.out.println(name + ": stage 1 " + sleep);
				Thread.sleep(sleep);
				// all threads need to wait for each other to get to this point
				System.out.println(name + ": 1st time waiting, " + barrier.getNumberWaiting() + " waiting threads");
				barrier.await();

				System.out.println(name + ": stage 2 " + sleep);
				Thread.sleep(sleep);
				// the barrier can be reused !!!
				System.out.println(name + ": 2nd time waiting, " + barrier.getNumberWaiting() + " waiting threads");
				barrier.await();

				System.out.println(name + ": finished");

			} catch (InterruptedException | BrokenBarrierException e) {
				e.printStackTrace();
			}
		}
	}
}

