package l10_concurrent;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class InvokeAll {

	record Result(String name, String timestamp) {

		@Override
		public String toString() {
			return "Result [name=" + name + ", value=" + timestamp + "]";
		}
	}

	record Task(String name) implements Callable<Result> {

		@Override
		public Result call() {
			System.out.printf("%s: Staring\n", name);
			int duration = RandomGenerators.getIntThread(8000);
			System.out.printf("%s: Waiting %d ms for results.\n", name, duration);
			ThreadHelperMethods.safeSleep(duration);
			return new Result(name, LocalTime.now().toString());
		}
	}

	public static void main(String[] args) {
		List<Future<Result>> resultList = null;
		List<Task> taskList = new ArrayList<>();
		for (int i = 0; i < 5; i++) {
			taskList.add(new Task("Task-" + i));
		}
		try (ExecutorService executor = Executors.newFixedThreadPool(2)) {
			//Execute all tasks and get reference to Future objects
			resultList = executor.invokeAll(taskList);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		System.out.println("\n========Printing the results======");
		assert resultList != null;
		for (Future<Result> future : resultList) {
			try {
				Result result = future.get();
				System.out.println(result.name() + ": " + result.timestamp());
			} catch (InterruptedException | ExecutionException e) {
				e.printStackTrace();
			}
		}
	}
}

