package l10_concurrent;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;

import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Callable;

public class InvokeAny {

	record UserValidatorTask(String source, String user, String password) implements Callable<String> {

		private boolean validate(String name, String password) {
			// Connect to backend based on source value
			// and validate the credentials
			int duration = RandomGenerators.getIntThread(2, 10);
			System.out.printf("%s : validating a user in %d seconds\n", source, duration);
			boolean valid = name.equals("some username") && password.equals("some password");
			try {
				TimeUnit.SECONDS.sleep(duration);
			} catch (InterruptedException e) {
				return false; // not an error; another method succeeded sooner
			}
			return valid;
		}

		@Override
		public String call() throws Exception {
			if (!validate(user, password)) {
				throw new Exception("Error validating user");
			}
			return source();
		}
	}

	public static void main(String[] args) throws InterruptedException {
		String username = "some username";
		String password = "some password";

		// Lightweight Directory Access Protocol
		UserValidatorTask ldapTask = new UserValidatorTask("LDAP",     username, password);
		// Data Base
		UserValidatorTask   dbTask = new UserValidatorTask("DataBase", username, password);
		List<UserValidatorTask> taskList = List.of(ldapTask, dbTask);

		// Suppose a new task comes in. If there is an idle thread waiting on the queue,
		// then the task producer hands off the task to that thread. Otherwise,
		// since the queue is always full, the executor creates a new thread to handle that task.
		try (ExecutorService executor = Executors.newCachedThreadPool()) {
			String result = executor.invokeAny(taskList);
			System.out.printf("User has been validated from : %s%n", result);
		} catch (InterruptedException | ExecutionException e) {
			e.printStackTrace();
		}
	}
}

