package l10_concurrent;

import utilities.RandomGenerators;
import utilities.ThreadHelperMethods;

import java.util.concurrent.Semaphore;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class PCSemaphore {
	private static final int numberOfProducers = 2;
	private static final int numberOfLoops = 5;

	public static void main(String[] args) {

		int n = Runtime.getRuntime().availableProcessors();
		Buffer sharedBuffer = new Buffer();

		try (ExecutorService executorService = Executors.newFixedThreadPool(n)) {
			for (int i = 0; i < numberOfProducers; i++) {
				executorService.submit(new Producer(sharedBuffer));
				executorService.submit(new Consumer(sharedBuffer));
			}
		}
	}

	static class Buffer {

		private int current = 0; // number of items currently in the buffer
		private final int bufferSize = 1; // size of buffer array
		private final int[] buffer = new int[bufferSize];
		// provides limited access to the buffer (mutual exclusion)
		private final Semaphore mutex = new Semaphore(1);
		// keep track of the number of empty elements in the array
		private final Semaphore empty = new Semaphore(bufferSize);
		// keep track of the number of filled elements in the array
		private final Semaphore full = new Semaphore(0);

		private void put(int item) {
			try {
				empty.acquire(); // this makes the producer stop running when buffer is full
				mutex.acquire(); // mutual exclusion
				buffer[current++] = item;
				System.out.format("Producer %3d put %3d%n", Thread.currentThread().threadId(), item);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} finally {
				mutex.release();
				full.release();
			}
		}

		private int get() {
			try {
				full.acquire(); // this makes the Consumer stop running when buffer is empty
				mutex.acquire(); // mutual exclusion
				int item = buffer[--current];
				System.out.format("\tConsumer %3d got %3d%n", Thread.currentThread().threadId(), item);
				return item;
			} catch (InterruptedException e) {
				e.printStackTrace();
				return -1;
			} finally {
				mutex.release();
				empty.release();
			}
		}
	}

	record Producer(Buffer buffer) implements Runnable {
		@Override
		public void run() {
			for (int loop = 0; loop < numberOfLoops; ++loop) {
				ThreadHelperMethods.safeSleep(2000);
				buffer.put(RandomGenerators.getIntThread(100));
			}
		}
	}

	record Consumer(Buffer buffer) implements Runnable {
		@Override
		public void run() {
			for (int loop = 0; loop < numberOfLoops; ++loop) {
				ThreadHelperMethods.safeSleep(2100);
				buffer.get();
			}
		}
	}
}

