package l10_concurrent;

public class RaceCondition {
	private static final int numberOfIncrements = 10000000;

	public static void main(String[] args) {
		Counter counter = new Counter();
		IncrementThread incrementThread = new IncrementThread(counter);
		DecrementThread decrementThread = new DecrementThread(counter);

		incrementThread.start();
		decrementThread.start();

		try {
			incrementThread.join();
			decrementThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			System.out.println("value = " + counter.get());
		}

	}

	static class Counter {
		private int count = 0;

		public void incrementAndGet() {
			++count;
		}

		public void decrementAndGet() {
			--count;
		}

		public int get() {
			return count;
		}
	}

	static class IncrementThread extends Thread {
		private final Counter counter;

		public IncrementThread(Counter counter) {
			this.counter = counter;
		}

		@Override
		public void run() {
			for (int i = 0; i < numberOfIncrements; ++i) {
				counter.incrementAndGet();
			}
		}
	}

	static class DecrementThread extends Thread {
		private final Counter counter;

		public DecrementThread(Counter counter) {
			this.counter = counter;
		}

		@Override
		public void run() {
			for (int i = 0; i < numberOfIncrements; ++i) {
				counter.decrementAndGet();
			}
		}
	}
}

