package l10_concurrent;

import java.util.concurrent.atomic.AtomicInteger;

//	Assume i is defined as int. The i++ (increment) operation it not an atomic
//	operation in Java. This also applies for the other numeric types, e.g. long.
//	etc).

//	The i++ operation first reads the value which is currently stored in i
//	(atomic operations) and then it adds one to it (atomic operation). But
//	between the read and the write the value of i might have changed.

//	Since Java 1.5 the java language provides atomic variables, e.g.
//	AtomicInteger or AtomicLong which provide methods like decrementAndGet(),
//	incrementAndGet() and getAndSet() which are atomic

public class RaceConditionAtomic {
	private static final int numberOfIncrements = 10000000;

	public static void main(String[] args) {
		AtomicInteger atomicCounter = new AtomicInteger(0);
		IncrementThread incrementThread = new IncrementThread(atomicCounter);
		DecrementThread decrementThread = new DecrementThread(atomicCounter);

		incrementThread.start();
		decrementThread.start();

		try {
			incrementThread.join();
			decrementThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			System.out.println("value = " + atomicCounter.get());
		}
	}

	static class IncrementThread extends Thread {
		private final AtomicInteger counter;

		public IncrementThread(AtomicInteger counter) {
			this.counter = counter;
		}

		@Override
		public void run() {
			for (int i = 0; i < numberOfIncrements; ++i) {
				counter.incrementAndGet();
			}
		}
	}

	static class DecrementThread extends Thread {
		private final AtomicInteger counter;

		public DecrementThread(AtomicInteger counter) {
			this.counter = counter;
		}

		@Override
		public void run() {
			for (int i = 0; i < numberOfIncrements; ++i) {
				counter.decrementAndGet();
			}
		}
	}
}


