package l10_concurrent;

import utilities.ThreadHelperMethods;

import java.util.concurrent.Semaphore;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/*
	 The java.util.concurrent.Semaphore class is a counting semaphore. That
	 means that it has two main methods:

	acquire()
	release()

	The counting semaphore is initialized with a given number of "permits". For
	each call to acquire() a permit is taken by the calling thread. For each
	call to release() a permit is returned to the semaphore. Thus, at most N
	threads can pass acquire() method without any release() calls, where N
	is the number of permits the semaphore was initialized with. The permits are
	just a simple counter.
*/

public class SemaphoreCountDown {
	public static void main(String[] args) {
		int n = Runtime.getRuntime().availableProcessors();
		//	true if this semaphore will guarantee first-in first-out granting of permits
		Semaphore semaphore = new Semaphore(1, true);
		try (ExecutorService executor = Executors.newFixedThreadPool(n)) {
			for (int i = 0; i < n; i++) {
				executor.submit(new Counter(i, semaphore));
			}
		}
	}

	record Counter(int number, Semaphore semaphore) implements Runnable {
		static int counter = 20;

		@Override
		public void run() {
			while (true) {
				//	sleeps BEFORE acquiring the semaphore
				ThreadHelperMethods.safeSleep(1000, 3000);
				try {
					semaphore.acquire();
					if (counter <= 0) {
						break;
					}
					counter--;
					System.out.println("Worker " + number + " counter " + counter);
				} catch (InterruptedException e) {
					e.printStackTrace();
				} finally {
					semaphore.release();
				}
			}
		}
	}
}

