package l10_concurrent;

import utilities.ThreadHelperMethods;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SimpleThreadPool {

	public static void main(String[] args) {

		final int n = (args.length > 0) ? Integer.parseInt(args[0]) :
			Runtime.getRuntime().availableProcessors();
		final int noWorkers = 2 * n;
		try (ExecutorService executor = Executors.newFixedThreadPool(n)) {
			for (int i = 0; i < noWorkers; i++) {
				executor.submit(new WorkerThread("Worker " + i));
			}
		}
		// The close() method shuts down an ExecutorService in two phases,
		// first by calling shutdown() to reject incoming tasks, and then calling shutdownNow(),
		// if necessary, to cancel any lingering tasks.
		// awaitTermination() blocks until all tasks have completed execution after a shutdown request.
		System.out.println("All threads finished");
	}

	record WorkerThread(String threadID) implements Runnable {

		@Override
		public void run() {
			System.out.println(threadID + " started");
			ThreadHelperMethods.safeSleep(5000);
			System.out.println(threadID + " done");
		}
	}
}

